/*
  Created: 03.19.05 11:34:57 by Attila Nagyidai

  $Id: C\040Console.c,v 1.1.2.1 2003/08/13 00:38:46 neum Exp $

  This file is part of IBSH (Iron Bars Shell) , a restricted Unix shell
  Copyright (C) 2005  Attila Nagyidai

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

  Author: Attila Nagyidai
  Email: na@ent.hu

  Co-Author: Shy
  Email: shy@cpan.org

  Co-Author: Witzy
  Email: weetzy@users.sourceforge.net
  
  URL: http://ibsh.sourceforge.net
  IRC: irc.freenode.net #ibsh
  RSS, Statistics, etc: http://sourceforge.net/projects/ibsh/

*/

/* Header files */
#include "ibsh.h"



/* Shy's improved version of the original (and not well working) loadconfig. */
/* Reads both config files, and parses the contents into arrays. */
/* This one effectively dismisses every comment from the files. */
/* Technical Description: */
/* Variables: file pointer, counters, temporary string arrays. */
/* The method is the same for both files. First open the file, catch */
/* any errors. Read file 'til eof. Not read comments (starting with '#'), */
/* remove trailing newline character. Copy the finished item to the */
/* pass-by-address arguments. */
void LoadConfig( void )
{
  FILE *fp;
  int i = 0;
  Strng tmp[MAX_ITEMS];


  /* Open the commands list file first. Read every item and */
  /* store in the array. No comments please. */
  fp = fopen(COMMANDS_LIST, "r");
  if ( fp == NULL ) {
      fprintf(stderr, "Can not open file %s: %s\n", COMMANDS_LIST,
      strerror(errno));      exit(0);
  }
  while ( !feof(fp) ) {
        fgets(tmp[i],STRING_SIZE,fp);
        if ( tmp[i][0] != '#' ) {
                // Delete '\n'
                tmp[i][strlen(tmp[i]) - 1] = '\0';
                strncpy(commands[i],tmp[i],strlen(tmp[i]));
                /*printf("COMMANDS %s\n",commands[i]);*/
                i++;
      }
  }
  fclose(fp);
}



/* Checks, if the user command, is blacklisted, a hack attempt, */
/* or it is a real and allowed command. */
/* Technical Description: */
/* Variables: pointer for the strtok, temporary strings, counters, */
/* and an integer to check, how deep is the user in the jail. That is */
/* the level of subdirectories below jail root. It is set to -1, because */
/* there is always a / in the jailpath. So only subdir /'s are counted. */
/* Check if the command contains special characters, if yes, quit. */
/* If the user is in jailroot, a ../ is not appropriate! */
/* Count the slashes in the jailpath, so if the user uses way too many */
/* ../ 's, then we will know. Split the command to particles by the spaces. */
/* Count the dirups (../); if a token starts with a /, paste the homedir path */
/* right in front of it. Thats your root booby, not / !!!! */
/* Finally check the command against the COMMANDS_LIST. */
int CommandOK( const char *thecommand, const char *rootdir,
const char *jailpath, char *newcommand )
{
  char *tok;
  char temp1[STRING_SIZE], *temp2;
  int i = 0, j = 0;
  int subdirlevel = -1; /* jailpath always starts with a / */
  int dirupfound = 0;
  int listed = 0;

  /* First, get the fancy stuff: */
  /* ../ out of the jailroot, too many ../ out of some */
  /* subdirectory in the jail, multiple commands, pipes. */
  strncpy(newcommand, "", LINE_SIZE);
  
  if ( (strstr(thecommand, ";")) != NULL ) {
      return 0;
  }
  if ( (strstr(thecommand, "|")) != NULL ) {
      return 0;
  }
  if ( (strstr(thecommand, "&&")) != NULL ) {
      return 0;
  }
  if ( (strstr(thecommand, "||")) != NULL ) {
      return 0;
  }
  /* The user is in the jailroot. */
  if ( (strcmp(jailpath, "/")) == 0 ) {
      /* Does the user wish to get out ? */
      if ( (strstr(thecommand, "..")) != NULL ) {
             return 0;
      }
  }
  /* The user is deeper, than the jailroot, and */
  /* this is a problem. How deep is he, how many */
  /* ../ do we allow ?? */
  else {
      for (i = 0; i < strlen(jailpath); i++) {
          if ( jailpath[i] == '/' ) {
              subdirlevel++;
          }
      }
  }

  /* Split the command */
  for (tok = strtok((void *) thecommand, " "); tok; tok = strtok(0, " ")) {
      /* Separate parts of the command with a space */
      if ( (strlen(newcommand)) > 0 ) {
          strncat(newcommand, " ", LINE_SIZE);
      }
      
      /* He wants to get to the real root, does he ? */
      /* In that case, add the jailroot to the left. */
      if ( tok[0] == '/' ) {
          strncat(newcommand, rootdir, STRING_SIZE);
      }
      
      /* how many ../ are here */
      /* if too many, that is more, then how deep */
      /* the user in the subdirs inside the jail is, */
      /* cancel the execution of the command. */
      if ( (strstr(tok, "../")) != NULL ) {
          strncpy(temp1, tok, LINE_SIZE);
          while (1) {
              temp2 = strstr(temp1, "../");
              if ( temp2 == NULL ) {
                  break;
              }
              LTrim3(temp2, temp1);
              dirupfound++;
          }
          if ( dirupfound > subdirlevel ) {
              return 0;
          }
          /* replace dirups with real path */
          for (i = 0; i < dirupfound; i++) {
              PathMinusOne(jailpath, tok, subdirlevel);
          }
      }
          /* if command is not listed, return 0 */
      i = 0;
      while ( ((strlen(commands[i])) > 0) && ( j == 0 ) ) {
          if ( (strcmp(tok, commands[i])) == 0 ) {
                listed = 1;
                break;
          }
          i++;
      }
      j++;
      strncat(newcommand, tok, STRING_SIZE);
  }
  return listed;
}


/* Counts the spaces in the command */
int nbspace(const char *command)
{
int i=0;
int nbspace=0;

while(command[i] != '\0'){
	if(command[i] == ' ')
		nbspace++;
	i++;
}

return nbspace;
}

/* Shy's improved and secured version of hhsystem, originally taken from */
/* the book: Linux Unix Systemprogramming by Helmut Herold. */
int  hhsystem(const char *user_command)   /*--- Version ohne Signalbehandlung ---*/
{
   pid_t   pid;
   int     status;
   int i=0;
   int find = 0;

   char *field;

   char path[STRING_SIZE];

   char *current_path;
   char *fieldspath;
   char *params[nbspace(user_command) + 1];

   DIR *currentdir;
   struct dirent *pdirent;

   if (user_command == NULL)
      return(1);   /* In Unix ist immer Kommandoprozessor vorhanden */

   if ( (pid=fork()) < 0)
      status = -1;

   else if (pid == 0) {
	 /* Split the command */
	field = strtok((char *)user_command," ");
	while(field != NULL){
		//printf("CHAMPS %s\n",field);
		params[i] = malloc(strlen(field) + 1);
		bzero(params[i],strlen(field)+1);
		strncpy(params[i],field,strlen(field));
		i++;
		field = strtok(NULL," ");

	}
	/* Put NULL at the end for execve */
	params[i] = NULL;

	/* Get PATH */
	current_path = getenv("PATH");

	/* Parse the PATH if the command is in the home dir it's skip !! */
        fieldspath = strtok((char *)current_path,":");
		while((fieldspath != NULL) && find != 1){
		//	printf("FIELD PATH %s\n",fieldsspath);
			if(!strstr(fieldspath,loggedin.udir)){
				currentdir = opendir(fieldspath);
				while(((pdirent = readdir(currentdir)) != NULL) && find != 1){
					if(!strcmp(pdirent->d_name,params[0])){
	//					printf("TROUVE %s!!!!\n",pdirent->d_name);
						find = 1;

					}
				}
				closedir(currentdir);
			}
			if(find == 0)
				fieldspath = strtok(NULL,":");
		}

	/* Contruct the real command with the good path */
	if(find == 1){
		memcpy(path,fieldspath,strlen(fieldspath));
        	path[strlen(fieldspath)] = '/';
        	strncat(path,params[0],strlen(params[0]));
		path[strlen(fieldspath) + strlen(params[0]) + 2] = '\0';

        	//printf("PATH FINAL %s %d ok!\n",path,strlen(path));
		//printf("PARAMS[0] %s\n",params[0]);
		//printf("PARAMS[1] %s\n",params[1]);

		execve(path,params,environ);
	}
	/* The command is in the home dir :( bad for you guys !! */
	else{
		status = -1;
	}
	_exit(127);

   } else
      while (waitpid(pid, &status, 0) < 0)
         if (errno != EINTR) {
            status = -1;
            break;
         }

   return(status);
}



