/*
  Created: 03.19.05 11:34:57 by Attila Nagyidai

  $Id: C\040Console.c,v 1.1.2.1 2003/08/13 00:38:46 neum Exp $

  This file is part of IBSH (Iron Bars Shell) , a restricted Unix shell
  Copyright (C) 2005  Attila Nagyidai

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

  Author: Attila Nagyidai
  Email: na@ent.hu

  Co-Author: Shy
  Email: shy@cpan.org

  Co-Author: Witzy
  Email: weetzy@users.sourceforge.net
  
  URL: http://ibsh.sourceforge.net
  IRC: irc.freenode.net #ibsh
  RSS, Statistics, etc: http://sourceforge.net/projects/ibsh/

*/

/* Header files */
#include "ibsh.h"

/* This is a builtin function of my text editor: Syn.*/
/* In this case, it is not used. */
void Usage(char *programName)
{
  fprintf(stderr, "%s usage:\n", programName);
  /* Modify here to add your usage message when the program is
   * called without arguments */
}

/* This is a builtin function of my text editor: Syn.*/
/* returns the index of the first argument that is not an option; i.e. */
/* does not start with a dash or a slash */
int HandleOptions(int argc, char *argv[])
{
  int i, firstnonoption = 0;

  for (i = 1; i < argc; i++) {
    if (argv[i][0] == '/' || argv[i][0] == '-') {
      switch (argv[i][1]) {
        /* An argument -? means help is requested */
        case '?':
          Usage(argv[0]);
          break;
        case 'h':
        case 'H':
          if (!strcmp(argv[i] + 1, "help")) {
            Usage(argv[0]);
            break;
          }
          /* If the option -h means anything else
           * in your application add code here
           * Note: this falls through to the default
           * to print an "unknow option" message
          */
        /* add your option switches here */
        default:
          fprintf(stderr, "unknown option %s\n", argv[i]);
          break;
      }
    }
    else {
      firstnonoption = i;
      break;
    }
  }
  return firstnonoption;
}



/* Remove the path of the Jail root dir from the displayed paths! */
/* Return the jail path from the absolute path. */
/* Copy characters from absolute path to jail path, starting, where the */
/* jail root ends. */
void GetPositionInJail( const char *abspath, const char *rootdir, char *relpath )
{
  int i = 0;
  int j = 0;
  
  bzero(relpath, strlen(relpath));
  for (i = strlen(rootdir); i < strlen(abspath); i++) {
      relpath[j] = abspath[i];
      j++;
  }
  relpath[j] = '\0';
}

/* Take 3 characters from left off a string. */
/* It practically removes one ../ . */
void LTrim3( const char *base, char *result )
{
  int i = 0;
  int j = 0;

  bzero(result, strlen(result));
  for (i = 3; i < strlen(base); i++) {
      result[j] = base[i];
      j++;
  }
  result[j] = '\0';
}


/* This is my version of scanf. It offers good protection against */
/* buffer overflow attacks. */
/* Technical Description: */
/* Variables: the char to read to from stdin, the sum of chars typed in, */
/* and an integer for length checking. */
/* Read characters from stdin in a loop, until Enter is pressed. */
/* The line size check wont stop accepting characters, but it will */
/* break long commands to LINE_SIZEd pieces. */
void myscanf( char *vptr )
{
    char c, chars[STRING_SIZE];
    int i = 0;

    while ( ((c = getchar()) != 10) && ( i < LINE_SIZE ) ) {
        chars[i] = c;
        /* the user is not allowed to pass long lines of trash to ibsh */
        i++;
    }
    chars[i] = '\0';
    strncpy(vptr, chars, LINE_SIZE);
}



/* Remove one subdirectory from the path in the argument. */
/* In case the user uses ../ 's in his command. */
/* Technical Description: */
/* Variables: string pointer for the strtok function, and an */
/* integer to stop the removing. */
/* Disassemble the path by the slashes. And glue the required parts */
/* together. Number of required parts = number of all parts - 1 . */
void PathMinusOne( const char *basepath, char *evalpath, int slashcount )
{
  char *tok;
  int j = 1;
  
  bzero(evalpath, strlen(evalpath));
  if ( slashcount == 1 ) {
      strncpy(evalpath, "/", 2);
  }
  else {
      for (tok = strtok((void *) basepath, "/"); tok; tok = strtok(0, "/")) {
          if ( j < slashcount ) {
              strncat(evalpath, tok, LINE_SIZE);
              strncat(evalpath, "/", LINE_SIZE);
          }
          j++;
      }
  }
}

/* If the command is not ok, there is a possible hack attempt. */
/* Can also be a typo, but we're not taking any chances. v0.2a */
void log_attempt( const char *username )
{
  char logmsg[STRING_SIZE];

  snprintf(logmsg, 50, "Possible hack attempt by %s.", username);
  
  openlog("ibsh", LOG_PID, LOG_AUTH);
  syslog(LOG_WARNING, logmsg);
  closelog();
}


/* Get the date and time for logging. */
void gettime( char *timestr )
{
  time_t thismoment;
  
  time(&thismoment);
  strncpy(timestr, ctime(&thismoment), LINE_SIZE);
}


